// get the config and package and stuff
import { fail } from './fail.js';
const validConfig = (e) => !!e &&
    typeof e === 'object' &&
    (e.exports === undefined || validExports(e['exports'])) &&
    (e.dialects === undefined || validDialects(e['dialects']));
const isDialect = (d) => d === 'commonjs' || d === 'esm';
const validDialects = (d) => !!d && Array.isArray(d) && !d.some(d => !isDialect(d));
const validExports = (e) => {
    if (!e)
        return false;
    if (typeof e !== 'object')
        return false;
    for (const [sub, exp] of Object.entries(e)) {
        if (sub !== '.' && !sub.startsWith('./')) {
            fail(`tshy.exports key must be "." or start with "./", got: ${sub}`);
            process.exit(1);
        }
        if (typeof exp === 'string') {
            e[sub] = addDot(exp);
            continue;
        }
        if (typeof exp !== 'object' || !exp || Array.isArray(exp)) {
            fail(`tshy.exports ${sub} value must be string or import/require object, ` +
                `got: ${JSON.stringify(exp)}`);
            process.exit(1);
        }
        const { import: i, require: r } = exp;
        if (!i && !e) {
            fail(`tshy.exports ${sub} needs require or import, ` +
                `got: ${JSON.stringify(exp)}`);
            process.exit(1);
        }
        if ((i !== undefined && typeof i !== 'string') ||
            (r !== undefined && typeof r !== 'string')) {
            fail(`tshy.exports ${sub} import/require must be strings, ` +
                `got: ${JSON.stringify(exp)}`);
            process.exit(1);
        }
        if ((i !== undefined && join(i).startsWith('src/')) ||
            (r !== undefined && join(r).startsWith('src/'))) {
            fail(`tshy.exports ${sub} in src/ must be string paths, ` +
                `got: ${JSON.stringify(exp)}`);
            process.exit(1);
        }
        e[sub] = {};
        if (e[sub].types)
            e[sub].types = addDot(e[sub].types);
        if (e[sub].import)
            e[sub].types = addDot(e[sub].import);
        if (e[sub].require)
            e[sub].types = addDot(e[sub].require);
    }
    if (e.dialects) {
        if (!validDialects(e.dialects)) {
            fail(`tshy.dialects must be array containing 'esm' and/or 'commonjs', ` +
                `got: ${JSON.stringify(e.dialects)}`);
        }
    }
    return true;
};
const addDot = (s) => `./${join(s)}`;
const getConfig = (pkg, sources) => {
    const tshy = validConfig(pkg.tshy) ? pkg.tshy : {};
    if (tshy.exports)
        return tshy;
    const e = {
        './package.json': './package.json',
    };
    for (const i of sources) {
        if (/^\.\/src\/index\.[^\.]+$/.test(i)) {
            e['.'] = i;
            break;
        }
    }
    pkg.tshy = tshy;
    tshy.exports = e;
    return tshy;
};
import { join } from 'path/posix';
import pkg from './package.js';
import sources from './sources.js';
const config = getConfig(pkg, sources);
export default config;
//# sourceMappingURL=config.js.map